/* -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *
 * The contents of this file are subject to the Netscape Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/NPL/
 *
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 *
 * The Original Code is mozilla.org code.
 *
 * The Initial Developer of the Original Code is Netscape
 * Communications Corporation.  Portions created by Netscape are
 * Copyright (C) 1998 Netscape Communications Corporation. All
 * Rights Reserved.
 *
 * Contributor(s): 
 */


/*
xp.h

This file is left around for backwards compatability.  Nothing new should
be added to this file.  Rather, add it to the client specific section or
the cross-platform specific area depending on what is appropriate.

-------------------------------------------------------------------------*/
#ifndef _XP_H_
#define _XP_H_

#include "xp_mcom.h"
#include "client.h"

#ifdef HEAPAGENT

#define MEM_DEBUG         1
#define DEFINE_NEW_MACRO  1
#include <heapagnt.h>

#endif /* HEAPAGENT */

#endif /* !_XP_H_ */   
																  
