/* Automatically generated file; do not edit */

#include "prtypes.h"

#include "prlink.h"

extern void Java_java_util_zip_Adler32_update_stub();
extern void Java_java_util_zip_Adler32_update1_stub();
extern void Java_java_util_zip_CRC32_update_stub();
extern void Java_java_util_zip_CRC32_update1_stub();
extern void Java_java_util_zip_Deflater_setDictionary_stub();
extern void Java_java_util_zip_Deflater_deflate_stub();
extern void Java_java_util_zip_Deflater_getAdler_stub();
extern void Java_java_util_zip_Deflater_getTotalIn_stub();
extern void Java_java_util_zip_Deflater_getTotalOut_stub();
extern void Java_java_util_zip_Deflater_reset_stub();
extern void Java_java_util_zip_Deflater_end_stub();
extern void Java_java_util_zip_Deflater_init_stub();
extern void Java_java_util_zip_Inflater_setDictionary_stub();
extern void Java_java_util_zip_Inflater_inflate_stub();
extern void Java_java_util_zip_Inflater_getAdler_stub();
extern void Java_java_util_zip_Inflater_getTotalIn_stub();
extern void Java_java_util_zip_Inflater_getTotalOut_stub();
extern void Java_java_util_zip_Inflater_reset_stub();
extern void Java_java_util_zip_Inflater_end_stub();
extern void Java_java_util_zip_Inflater_init_stub();

PRStaticLinkTable zip_nodl_tab[] = {
  { "Java_java_util_zip_Adler32_update_stub", Java_java_util_zip_Adler32_update_stub },
  { "Java_java_util_zip_Adler32_update1_stub", Java_java_util_zip_Adler32_update1_stub },
  { "Java_java_util_zip_CRC32_update_stub", Java_java_util_zip_CRC32_update_stub },
  { "Java_java_util_zip_CRC32_update1_stub", Java_java_util_zip_CRC32_update1_stub },
  { "Java_java_util_zip_Deflater_setDictionary_stub", Java_java_util_zip_Deflater_setDictionary_stub },
  { "Java_java_util_zip_Deflater_deflate_stub", Java_java_util_zip_Deflater_deflate_stub },
  { "Java_java_util_zip_Deflater_getAdler_stub", Java_java_util_zip_Deflater_getAdler_stub },
  { "Java_java_util_zip_Deflater_getTotalIn_stub", Java_java_util_zip_Deflater_getTotalIn_stub },
  { "Java_java_util_zip_Deflater_getTotalOut_stub", Java_java_util_zip_Deflater_getTotalOut_stub },
  { "Java_java_util_zip_Deflater_reset_stub", Java_java_util_zip_Deflater_reset_stub },
  { "Java_java_util_zip_Deflater_end_stub", Java_java_util_zip_Deflater_end_stub },
  { "Java_java_util_zip_Deflater_init_stub", Java_java_util_zip_Deflater_init_stub },
  { "Java_java_util_zip_Inflater_setDictionary_stub", Java_java_util_zip_Inflater_setDictionary_stub },
  { "Java_java_util_zip_Inflater_inflate_stub", Java_java_util_zip_Inflater_inflate_stub },
  { "Java_java_util_zip_Inflater_getAdler_stub", Java_java_util_zip_Inflater_getAdler_stub },
  { "Java_java_util_zip_Inflater_getTotalIn_stub", Java_java_util_zip_Inflater_getTotalIn_stub },
  { "Java_java_util_zip_Inflater_getTotalOut_stub", Java_java_util_zip_Inflater_getTotalOut_stub },
  { "Java_java_util_zip_Inflater_reset_stub", Java_java_util_zip_Inflater_reset_stub },
  { "Java_java_util_zip_Inflater_end_stub", Java_java_util_zip_Inflater_end_stub },
  { "Java_java_util_zip_Inflater_init_stub", Java_java_util_zip_Inflater_init_stub },
  { 0, 0, },
};
